@@ -1,11 +1,11 @@
 #
 # port.rb
 #
-#   Copyright (c) 1998-2002 Minero Aoki <aamine@loveruby.net>
+# Copyright (c) 1998-2003 Minero Aoki <aamine@loveruby.net>
 #
-#   This program is free software.
-#   You can distribute/modify this program under the terms of
-#   the GNU Lesser General Public License version 2 or later.
+# This program is free software.
+# You can distribute/modify this program under the terms of
+# the GNU Lesser General Public License version 2 or later.
 #
 
 require 'tmail/stringio'
@@ -14,12 +14,14 @@ require 'tmail/stringio'
 module TMail
 
   class Port
-    # to allow user extention
+    def reproducible?
+      false
+    end
   end
 
 
   ###
-  ###  FilePort
+  ### FilePort
   ###
 
   class FilePort < Port
@@ -34,7 +36,7 @@ module TMail
     alias ident filename
 
     def ==( other )
-      other.respond_to? :filename and @filename == other.filename
+      other.respond_to?(:filename) and @filename == other.filename
     end
 
     alias eql? ==
@@ -44,7 +46,11 @@ module TMail
     end
 
     def inspect
-      "#<#{type}:#{@filename}>"
+      "#<#{self.class}:#{@filename}>"
+    end
+
+    def reproducible?
+      true
     end
 
     def size
@@ -53,15 +59,15 @@ module TMail
 
 
     def ropen( &block )
-      File.open( @filename, &block )
+      File.open(@filename, &block)
     end
 
     def wopen( &block )
-      File.open( @filename, 'w', &block )
+      File.open(@filename, 'w', &block)
     end
 
     def aopen( &block )
-      File.open( @filename, 'a', &block )
+      File.open(@filename, 'a', &block)
     end
 
 
@@ -88,12 +94,12 @@ module TMail
     alias mv move_to
 
     def copy_to( port )
-      if FilePort === port then
+      if FilePort === port
         copy_file @filename, port.filename
       else
         File.open(@filename) {|r|
         port.wopen {|w|
-            while s = r.sysread(4096) do
+            while s = r.sysread(4096)
               w.write << s
             end
         } }
@@ -108,11 +114,11 @@ module TMail
     def copy_file( src, dest )
       st = r = w = nil
 
-      File.open( src,  'rb' ) {|r|
-      File.open( dest, 'wb' ) {|w|
+      File.open(src,  'rb') {|r|
+      File.open(dest, 'wb') {|w|
           st = r.stat
           begin
-            while true do
+            while true
               w.write r.sysread(st.blksize)
             end
           rescue EOFError
@@ -170,7 +176,7 @@ module TMail
     def set_status( tag, flag )
       begin
         tmpfile = @filename + '.tmailtmp.' + $$.to_s
-        File.open( tmpfile, 'w' ) {|f|
+        File.open(tmpfile, 'w') {|f|
           write_status f, tag, flag
         }
         File.unlink @filename
@@ -182,11 +188,11 @@ module TMail
 
     def write_status( f, tag, flag )
       stat = ''
-      File.open( @filename ) {|r|
-        while line = r.gets do
-          if line.strip.empty? then
+      File.open(@filename) {|r|
+        while line = r.gets
+          if line.strip.empty?
             break
-          elsif m = /\AX-Delta-Status:/i.match(line) then
+          elsif m = /\AX-TMail-Status:/i.match(line)
             stat = m.post_match.strip
           else
             f.print line
@@ -194,20 +200,20 @@ module TMail
         end
 
         s = procinfostr(stat, tag, flag)
-        f.puts 'X-Delta-Status: ' + s unless s.empty?
+        f.puts 'X-TMail-Status: ' + s unless s.empty?
         f.puts
 
-        while s = r.read(2048) do
+        while s = r.read(2048)
           f.write s
         end
       }
     end
 
     def get_status( tag )
-      File.foreach( @filename ) {|line|
+      File.foreach(@filename) {|line|
         return false if line.strip.empty?
-        if m = /\AX-Delta-Status:/i.match(line) then
-          return m.post_match.strip.include? tag[0]
+        if m = /\AX-TMail-Status:/i.match(line)
+          return m.post_match.strip.include?(tag[0])
         end
       }
       false
@@ -218,6 +224,24 @@ module TMail
 
   class MaildirPort < FilePort
 
+    def move_to_new
+      new = replace_dir(@filename, 'new')
+      File.rename @filename, new
+      @filename = new
+    end
+
+    def move_to_cur
+      new = replace_dir(@filename, 'cur')
+      File.rename @filename, new
+      @filename = new
+    end
+
+    def replace_dir( path, dir )
+      "#{File.dirname File.dirname(path)}/#{dir}/#{File.basename path}"
+    end
+    private :replace_dir
+
+
     include MailFlags
 
     private
@@ -225,13 +249,13 @@ module TMail
     MAIL_FILE = /\A(\d+\.[\d_]+\.[^:]+)(?:\:(\d),(\w+)?)?\z/
 
     def set_status( tag, flag )
-      unless m = MAIL_FILE.match(File.basename @filename) then
-        newname = @filename + ':2,' + tag
-      else
+      if m = MAIL_FILE.match(File.basename(@filename))
         s, uniq, type, info, = m.to_a
         return if type and type != '2'  # do not change anything
         newname = File.dirname(@filename) + '/' +
                   uniq + ':2,' + procinfostr(info.to_s, tag, flag)
+      else
+        newname = @filename + ':2,' + tag
       end
 
       File.link @filename, newname
@@ -240,8 +264,8 @@ module TMail
     end
 
     def get_status( tag )
-      m = MAIL_FILE.match(File.basename @filename) or return false
-      m[2] == '2' and m[3].to_s.include? tag[0]
+      m = MAIL_FILE.match(File.basename(@filename)) or return false
+      m[2] == '2' and m[3].to_s.include?(tag[0])
     end
   
   end
@@ -283,26 +307,28 @@ module TMail
     end
 
     def inspect
-      "#<#{type}:id=#{sprintf '0x%x', @buffer.id}>"
+      "#<#{self.class}:id=#{sprintf '0x%x', @buffer.id}>"
     end
 
+    def reproducible?
+      true
+    end
 
     def ropen( &block )
       @buffer or raise Errno::ENOENT, "#{inspect} is already removed"
-      StringInput.open( @buffer, &block )
+      StringInput.open(@buffer, &block)
     end
 
     def wopen( &block )
       @buffer = ''
-      StringOutput.new( @buffer, &block )
+      StringOutput.new(@buffer, &block)
     end
 
     def aopen( &block )
       @buffer ||= ''
-      StringOutput.new( @buffer, &block )
+      StringOutput.new(@buffer, &block)
     end
 
-
     def remove
       @buffer = nil
     end
@@ -318,7 +344,7 @@ module TMail
     alias cp copy_to
 
     def move_to( port )
-      if StringPort === port then
+      if StringPort === port
         str = @buffer
         port.instance_eval { @buffer = str }
       else
